package cn.ibizlab.codegen.model;

import net.ibizsys.model.dataentity.IPSDataEntity;
import net.ibizsys.model.dataentity.logic.IPSDELogic;
import net.ibizsys.model.dataentity.logic.IPSDELogicNode;
import net.ibizsys.model.dataentity.logic.IPSDELogicParam;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class LogicModel extends BaseModel {

    private EntityModel entity;

    private boolean hasStartWF = false;
    private List<String> ruleGlobalParams = new ArrayList<>();

    public List<String> getRuleGlobalParams() {
        return ruleGlobalParams;
    }

    public LogicModel(EntityModel entityModel, IPSDELogic iPSDELogic) {
        this.opt = iPSDELogic;
        this.entity = entityModel;
        this.setCodeName(iPSDELogic.getCodeName());
        this.setName(iPSDELogic.getName());
        this.setId(String.format("%1$s-%2$s", entity.getCodeName(), iPSDELogic.getCodeName()));
        if (!CollectionUtils.isEmpty(iPSDELogic.getPSDELogicParams())) {
            for (IPSDELogicParam iPSDELogicParam : iPSDELogic.getPSDELogicParams()) {
                String strParamName = iPSDELogicParam.getCodeName().toLowerCase();
                if (iPSDELogicParam.getParamPSDataEntity() != null) {
                    String strDataType = String.format("%s.core.%s.domain.%s",
                            entityModel.getSystem().getPackageName(),
                            iPSDELogicParam.getParamPSDataEntity().getPSSystemModule().getCodeName().toLowerCase(),
                            iPSDELogicParam.getParamPSDataEntity().getCodeName()
                    );
                    ruleGlobalParams.add(String.format("global %s %s;", strDataType, strParamName));
                } else {
                    String strDataType = "java.util.Map";
                    ruleGlobalParams.add(String.format("global %s %s;", strDataType, strParamName));
                }
            }
        }
        if (!CollectionUtils.isEmpty(iPSDELogic.getPSDELogicNodes())) {
            Map<String, String> map = new HashMap<>();
            for (IPSDELogicNode iPSDELogicNode : iPSDELogic.getPSDELogicNodes()) {
                if (iPSDELogicNode.getLogicNodeType().equals("STARTWF")) {
                    hasStartWF = true;
                }
                try {
                    Method method = iPSDELogicNode.getClass().getDeclaredMethod("getDstPSDataEntity");
                    IPSDataEntity dstPSDataEntity = (IPSDataEntity) method.invoke(iPSDELogicNode);
                    if (!map.containsKey(dstPSDataEntity.getName())) {
                        String strDataType = String.format("%s.core.%s.service.I%sService",
                                entityModel.getSystem().getPackageName(),
                                dstPSDataEntity.getPSSystemModule().getCodeName().toLowerCase(),
                                dstPSDataEntity.getCodeName()
                        );
                        ruleGlobalParams.add(String.format("global %s %s;", strDataType, String.format("%sservice;",dstPSDataEntity.getCodeName().toLowerCase())));
                        map.put(dstPSDataEntity.getName(), dstPSDataEntity.getName());
                    }
                } catch (Exception e) {

                }
            }
        }
        if (hasStartWF) {
            String strDataType = "cn.ibizlab.util.client";
            ruleGlobalParams.add(String.format("global %s %s;", strDataType, "wfClient"));
        }
        if (true) {
            String strDataType = String.format("%s.core.%s.service.I%sService",
                    entityModel.getSystem().getPackageName(),
                    entityModel.getDataEntity().getPSSystemModule().getCodeName().toLowerCase(),
                    entityModel.getDataEntity().getCodeName()
            );
            ruleGlobalParams.add(String.format("global %s %s;", strDataType, String.format("iBzSys%sDefaultService;",entityModel.getCodeName())));
        }
        ruleGlobalParams.add(String.format("global %s %s;", "cn.ibizlab.util.security.AuthenticationUser", "curuser"));
    }

    public IPSDELogic getPSDELogic() {
        return (IPSDELogic) opt;
    }

}
